module R10K
  # When updating to a new major (X) or minor (Y) version, include `#major` or
  # `#minor` (respectively) in your commit message to trigger the appropriate
  # release. Otherwise, a new patch (Z) version will be released.
  VERSION = '5.0.3'
end
