cachedir: '/var/cache/r10k'
git:
  provider: '<%= git_provider %>'
sources:
  <% for source in sources %>
    <%= source.repo_name %>:
      basedir: "<%= env_path %>"
      remote: "<%= source.control_remote %>"
  <% end %>
